# -*- coding: utf-8 -*-
#
# satcomum/util.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import locale


def digitos(valor):
    """Resulta em uma string contendo apenas os dígitos da string original.

    .. sourcecode:: python

        >>> digitos('teste')
        ''
        >>> digitos('2015-08-20')
        '20150820'

    """
    return ''.join([d for d in valor if d.isdigit()])


def texto_decimal(valor, remover_zeros=True):
    """Converte um valor :py:class:`decimal.Decimal` para texto, com a opção de
    remover os zeros à direita não significativos. A conversão para texto irá
    considerar o :py:module:`locale` para converter o texto pronto para
    apresentação.

    :param decimal.Decimal valor: Valor a converter para texto.
    :param bool remover_zeros: *Opcional* Indica se os zeros à direita não
        significativos devem ser removidos do texto, o que irá incluir o
        separador decimal se for o caso.

    .. sourcecode:: python

        >>> from decimal import Decimal
        >>> valor = Decimal('10.0000')
        >>> texto_decimal(valor)
        '10'
        >>> texto_decimal(valor, remover_zeros=False)
        '10.0000'
        >>> texto_decimal(Decimal('10.00100'))
        '10.001'
        >>> texto_decimal(Decimal('-1.00'))
        '-1'
        >>> texto_decimal(Decimal('-0.010'))
        '-0.01'

    """
    texto = '{:n}'.format(valor)
    if remover_zeros:
        dp = locale.localeconv().get('decimal_point')
        texto = texto.rstrip('0').rstrip(dp) if dp in texto else texto
    return texto


def forcar_unicode(arg):
    """Força uma conversão do argumento para unicode, se necessário.

    .. sourcecode:: python

        >>> forcar_unicode('teste')
        u'teste'
        >>> forcar_unicode(u'teste')
        u'teste'
        >>> forcar_unicode(1)
        Traceback (most recent call last):
         ...
        TypeError: ...

    """
    if isinstance(arg, unicode):
        return arg
    elif isinstance(arg, str):
        return arg.decode('utf-8')

    raise TypeError(
            'Expected unicode or str; got {!r} ({!r})'.format(arg, type(arg)))


def modulo11(base):
    """Calcula o dígito verificador (DV) para o argumento usando "Módulo 11".

    :param str base: String contendo os dígitos sobre os quais o DV será
        calculado, assumindo que o DV não está incluído no argumento.

    :return: O dígito verificador calculado.
    :rtype: int

    .. sourcecode:: python

        >>> modulo11('0')  # digito resultante: 11
        0
        >>> modulo11('6')  # digito resultante: 10
        0
        >>> modulo11('1')  # digito resultante: 9
        9

        # chaves de CF-e-SAT emitidos em ambiente de testes
        # CFe35150808723218000186599000040190000241114257
        # CFe35150808723218000186599000040190000253347537
        >>> modulo11('3515080872321800018659900004019000024111425')
        7
        >>> modulo11('3515080872321800018659900004019000025334753')
        7

    """
    pesos = '23456789' * ((len(base) / 8) + 1)
    acumulado = sum([int(a) * int(b) for a, b in zip(base[::-1], pesos)])
    digito = 11 - (acumulado % 11)
    return 0 if digito >= 10 else digito


def validar_casas_decimais(valor, minimo=1, maximo=2):
    """Valida o número de casas decimais. Se o número de casas decimais não
    estiver dentro do mínimo e máximo, será lançada uma exceção do tipo
    :py:exc:`ValueError`.

    :param valor: Um objeto :py:class:`~decimal.Decimal`.

    :param minimo: Valor inteiro maior ou igual a zero indicando o número
        mínimo de casas decimais. Se não informado, ``1`` é o mínimo.

    :param maximo: Valor inteiro maior ou igual a zero indicando o número
        máximo de casas decimais. Se não informado, ``2`` é o máximo.

    .. sourcecode:: python

        >>> from decimal import Decimal
        >>> validar_casas_decimais(Decimal('0.1'))
        >>> validar_casas_decimais(Decimal('0.12'))
        >>> validar_casas_decimais(Decimal('0'), minimo=0)
        >>> validar_casas_decimais(Decimal('0.1230'), maximo=4)

        >>> validar_casas_decimais(Decimal('1.001'))
        Traceback (most recent call last):
         ...
        ValueError: Numero de casas decimais fora dos limites esperados (valor=Decimal('1.001'), minimo=1, maximo=2): DecimalTuple(sign=0, digits=(1, 0, 0, 1), exponent=-3)

        >>> validar_casas_decimais(Decimal('1'))
        Traceback (most recent call last):
         ...
        ValueError: Numero de casas decimais fora dos limites esperados (valor=Decimal('1'), minimo=1, maximo=2): DecimalTuple(sign=0, digits=(1,), exponent=0)


    :raises ValueError: Se o valor possuir um número de casas decimais fora dos
        limites mínimo e máximo informados.

    """
    atributos = valor.as_tuple()
    if not (minimo <= abs(atributos.exponent) <= maximo):
        raise ValueError('Numero de casas decimais fora dos limites esperados '
                '(valor={!r}, minimo={!r}, maximo={!r}): {!r}'.format(
                        valor, minimo, maximo, atributos))



# -*- coding: utf-8 -*-
#
# satcfe/constantes.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

VERSAO_ER = '2.19.07'
"""Versão da Especificação de Requisitos (ER) do SAT-CF-e na qual esta
implementação se baseia.
"""

VERSAO_LAYOUT_ARQUIVO_DADOS_AC = '0.07'
"""Versão do layout do arquivo de dados enviado pelo Aplicativo Comercial."""

XML_DECL = '<?xml version="1.0" ?>'
"""Declaração de documento XML."""

XML_DECL_UNICODE = u'<?xml version="1.0" encoding="utf-8" ?>'
"""Declaração de documento XML incluindo a codificação de texto UTF-8."""

ROOT_TAG_VENDA = 'CFe'
"""Nome da tag raiz para documentos CF-e-SAT de venda."""

ROOT_TAG_CANCELAMENTO = 'CFeCanc'
"""Nome da tag raiz para documentos CF-e-SAT de cancelamento."""

ASSINATURA_AC_TESTE = 'SGR-SAT SISTEMA DE GESTAO E RETAGUARDA DO SAT'
"""Assinatura do Aplicativo Comercial utilizada nos documentos de venda e
cancelamento realizados contra equipamentos SAT para desenvolvimento.
"""


STANDARD_C = 1
WINDOWS_STDCALL = 2

CONVENCOES_CHAMADA = (
        (STANDARD_C, u'Standard C'),
        (WINDOWS_STDCALL, u'Windows "stdcall"'),)


CERTIFICADO_ACSAT_SEFAZ = 1
CERTIFICADO_ICPBRASIL = 2
CERTIFICADO_ICPBRASIL_RENOVACAO = 3

TIPOS_CERTIFICADOS = (
        (CERTIFICADO_ACSAT_SEFAZ, u'AC-SAT/SEFAZ'),
        (CERTIFICADO_ICPBRASIL, u'ICP-Brasil'),
        (CERTIFICADO_ICPBRASIL_RENOVACAO, u'Renovação Certificado ICP-Brasil'),
    )


CODIGO_ATIVACAO_REGULAR = 1
CODIGO_ATIVACAO_EMERGENCIA = 2

TIPOS_CODIGOS_ATIVACAO = (
        (CODIGO_ATIVACAO_REGULAR, u'Código de Ativação Regular'),
        (CODIGO_ATIVACAO_EMERGENCIA, u'Código de Ativação de Emergência'),
    )


# tpAmb (B10), identificação do ambiente
B10_PRODUCAO = '1'
B10_TESTES = '2'

B10_TPAMB = (
        (B10_PRODUCAO, u'Produção'),
        (B10_TESTES, u'Testes'),
    )


# emit (C01), valores do elemento 'cRegTribISSQN' (C15)
C15_MICROEMPRESA_MUNICIPAL = '1'
C15_ESTIMATIVA = '2'
C15_SOCIEDADE_PROFISSIONAIS = '3'
C15_COOPERATIVA = '4'
C15_MICROEMPRESARIO_INDIVIDUAL = '5'

C15_CREGTRIBISSQN_EMIT = (
        (C15_MICROEMPRESA_MUNICIPAL, u'Microempresa Municipal'),
        (C15_ESTIMATIVA, u'Estimativa'),
        (C15_SOCIEDADE_PROFISSIONAIS, u'Sociedade de Profissionais'),
        (C15_COOPERATIVA, u'Cooperativa'),
        (C15_MICROEMPRESARIO_INDIVIDUAL, u'Microempresario Individual (MEI)'),
    )


# emit (C01), valores do elemento 'indRatISSQN' (C16)
C16_RATEADO = 'S'
C16_NAO_RATEADO = 'N'

C16_INDRATISSQN_EMIT = (
        (C16_RATEADO, u'Rateado'),
        (C16_NAO_RATEADO, u'Não Rateado'),
    )


I11_ARREDONDAMENTO = 'A'
I11_TRUNCAMENTO = 'T'

I11_INDREGRA = (
        (I11_ARREDONDAMENTO, u'Arredondamento'),
        (I11_TRUNCAMENTO, u'Truncamento'),)


N06_NACIONAL = '0'
N06_ESTRANGEIRA_IMPORTACAO_DIRETA = '1'
N06_ESTRANGEIRA_ADQUIRIDA_MERCADO_INTERNO = '2'
N06_NACIONAL_CONTEUDO_IMPORTADO_41_70_PORCENTO = '3'
N06_NACIONAL_PROCESSOS_PRODUTIVOS_BASICOS = '4'
N06_NACIONAL_CONTEUDO_IMPORTADO_MENOR_40_PORCENTO = '5'
N06_ESTRANGEIRA_IMPORTACAO_DIRETA_CAMEX = '6'
N06_ESTRANGEIRA_ADQUIRIDA_MERCADO_INTERNO_CAMEX = '7'
N06_NACIONAL_CONTEUDO_IMPORTADO_SUPERIOR_70_PORCENTO = '8'

N06_ORIG = (
        (N06_NACIONAL, u'Nacional'),
        (N06_ESTRANGEIRA_IMPORTACAO_DIRETA, u'Estrangeira, importação direta'),
        (N06_ESTRANGEIRA_ADQUIRIDA_MERCADO_INTERNO, u'Estrangeira, adquirida no mercado interno'),
        (N06_NACIONAL_CONTEUDO_IMPORTADO_41_70_PORCENTO, u'Nacional, conteúdo importado entre 41% e 70%'),
        (N06_NACIONAL_PROCESSOS_PRODUTIVOS_BASICOS, u'Nacional, produção conforme processos produtivos básicos'),
        (N06_NACIONAL_CONTEUDO_IMPORTADO_MENOR_40_PORCENTO, u'Nacional, conteúdo importado inferior ou igual a 40%'),
        (N06_ESTRANGEIRA_IMPORTACAO_DIRETA_CAMEX, u'Estrangeira, importação direta, sem similar nacional, constante na lista da CAMEX e gás natural'),
        (N06_ESTRANGEIRA_ADQUIRIDA_MERCADO_INTERNO_CAMEX, u'Estrangeira, adquirida no mercado interno, sem similar nacional, constante na lista da CAMEX e gás natural'),
        (N06_NACIONAL_CONTEUDO_IMPORTADO_SUPERIOR_70_PORCENTO, u'Nacional, conteúdo importado superior a 70%'),
    )


# ICMS00 (N02), valores do elemento 'CST' (N07)
N07_TRIBUTADA_INTEGRALMENTE = '00'
N07_TRIBUTADA_COM_REDUCAO_BC = '20'
N07_OUTROS = '90'

N07_CST_ICMS00 = (
        (N07_TRIBUTADA_INTEGRALMENTE, u'Tributada integralmente'),
        (N07_TRIBUTADA_COM_REDUCAO_BC, u'Tributada com redução na base de cálculo'),
        (N07_OUTROS, u'Outros'),
    )


# ICMS40 (N03), valores do elemento 'CST' (N07)
N07_ISENTO = '40'
N07_NAO_TRIBUTADO = '41'
N07_ICMS_COBRADO_ANTERIORMENTE_ST = '60'

N07_CST_ICMS40 = (
        (N07_ISENTO, u'Isento'),
        (N07_NAO_TRIBUTADO, u'Não tributado'),
        (N07_ICMS_COBRADO_ANTERIORMENTE_ST, u'ICMS cobrado anteriormente por Substituição Tributária'),
    )


# ICMSSN102 (N04), valores do elemento 'CSOSN' (N10)
N10_TRIBUTADA_SN_SEM_PERMISSAO_CREDITO = '102'
N10_IMUNE = '300'
N10_NAO_TRIBUTADA = '400'
N10_ICMS_COBRADO_ANTERIORMENTE_ST = '500'

N10_CSOSN_ICMSSN102 = (
        (N10_TRIBUTADA_SN_SEM_PERMISSAO_CREDITO, u'Tributada pelo Simples Nacional sem permissão de crédito'),
        (N10_IMUNE, u'Imune'),
        (N10_NAO_TRIBUTADA, u'Não tributado'),
        (N10_ICMS_COBRADO_ANTERIORMENTE_ST, u'ICMS cobrado anteriormente por Substituição Tributária'),
    )


# ICMSSN900 (N05), valores do elemento 'CSOSN' (N10)
N10_OUTROS = '900'

N10_CSOSN_ICMSSN900 = (
        (N10_OUTROS, u'Outros'),
    )


# PISAliq (Q02), valores do elemento 'CST' (Q07)
Q07_OP_TRIBUTAVEL_ALIQUOTA_NORMAL = '01'
Q07_OP_TRIBUTAVEL_ALIQUOTA_DIFERENCIADA = '02'
Q07_OP_TRIBUTAVEL_ST = '05'

Q07_CST_PISALIQ = (
        (Q07_OP_TRIBUTAVEL_ALIQUOTA_NORMAL, u'Operação tributável (alíquota normal)'),
        (Q07_OP_TRIBUTAVEL_ALIQUOTA_DIFERENCIADA, u'Operação tributável (aliquota diferenciada)'),
        (Q07_OP_TRIBUTAVEL_ST, u'Operação tributável por Substituição Tributária'),
    )


# PISQtde (Q03), valores do elemento 'CST' (Q07)
Q07_OP_TRIBUTAVEL_BC_QTD_VENDIDA_X_UN_PROD = '03'

Q07_CST_PISQTDE = (
        (Q07_OP_TRIBUTAVEL_BC_QTD_VENDIDA_X_UN_PROD,
                u'Operação tributável (BC = qtd. vendida X un. de produto)'),
    )


# PISNT (Q04), valores do elemento 'CST' (Q07)
Q07_OP_TRIBUTAVEL_MONOFASICA_ALIQUOTA_ZERO = '04'
Q07_OP_TRIBUTAVEL_ALIQUOTA_ZERO = '06'
Q07_OP_ISENTA_CONTRIBUICAO = '07'
Q07_OP_SEM_INCIDENCIA_CONTRIBUICAO = '08'
Q07_OP_COM_SUSPENSAO_CONTRIBUICAO = '09'

Q07_CST_PISNT = (
        (Q07_OP_TRIBUTAVEL_MONOFASICA_ALIQUOTA_ZERO, u'Operação tributável (monofásica, alíquota zero)'),
        (Q07_OP_TRIBUTAVEL_ALIQUOTA_ZERO, u'Operação tributável (alíquota zero)'),
        (Q07_OP_ISENTA_CONTRIBUICAO, u'Operação isenta da contribuição'),
        (Q07_OP_SEM_INCIDENCIA_CONTRIBUICAO, u'Operação sem incidência da contribuição'),
        (Q07_OP_COM_SUSPENSAO_CONTRIBUICAO, u'Operação com suspensão da contribuição'),
    )


# PISSN (Q05), valores do elemento 'CST' (Q07)
Q07_OUTRAS_OPERACOES_SAIDA = '49'

Q07_CST_PISSN = (
        (Q07_OUTRAS_OPERACOES_SAIDA, u'Outras operações de saída'),
    )


# PISOutr (Q06), valores do elemento 'CST' (Q07)
Q07_OUTRAS_OPERACOES = '99'

Q07_CST_PISOUTR = (
        (Q07_OUTRAS_OPERACOES, u'Outras operações'),
    )


# COFINSAliq (S02), valores do elemento 'CST' (S07)
S07_OP_TRIBUTAVEL_ALIQUOTA_NORMAL = '01'
S07_OP_TRIBUTAVEL_ALIQUOTA_DIFERENCIADA = '02'
S07_OP_TRIBUTAVEL_ST = '05'

S07_CST_COFINSALIQ = (
        (S07_OP_TRIBUTAVEL_ALIQUOTA_NORMAL, u'Operação tributável (alíquota normal)'),
        (S07_OP_TRIBUTAVEL_ALIQUOTA_DIFERENCIADA, u'Operação tributável (aliquota diferenciada)'),
        (S07_OP_TRIBUTAVEL_ST, u'Operação tributável por Substituição Tributária'),
    )


# COFINSQtde (S03), valores do elemento 'CST' (S07)
S07_OP_TRIBUTAVEL_BC_QTD_VENDIDA_X_UN_PROD = '03'

S07_CST_COFINSQTDE = (
        (S07_OP_TRIBUTAVEL_BC_QTD_VENDIDA_X_UN_PROD,
                u'Operação tributável (BC = qtd. vendida X un. de produto)'),
    )


# COFINSNT (S04), valores do elemento 'CST' (S07)
S07_OP_TRIBUTAVEL_MONOFASICA_ALIQUOTA_ZERO = '04'
S07_OP_TRIBUTAVEL_ALIQUOTA_ZERO = '06'
S07_OP_ISENTA_CONTRIBUICAO = '07'
S07_OP_SEM_INCIDENCIA_CONTRIBUICAO = '08'
S07_OP_COM_SUSPENSAO_CONTRIBUICAO = '09'

S07_CST_COFINSNT = (
        (S07_OP_TRIBUTAVEL_MONOFASICA_ALIQUOTA_ZERO, u'Operação tributável (monofásica, alíquota zero)'),
        (S07_OP_TRIBUTAVEL_ALIQUOTA_ZERO, u'Operação tributável (alíquota zero)'),
        (S07_OP_ISENTA_CONTRIBUICAO, u'Operação isenta da contribuição'),
        (S07_OP_SEM_INCIDENCIA_CONTRIBUICAO, u'Operação sem incidência da contribuição'),
        (S07_OP_COM_SUSPENSAO_CONTRIBUICAO, u'Operação com suspensão da contribuição'),
    )


# COFINSSN (S05), valores do elemento 'CST' (S07)
S07_OUTRAS_OPERACOES_SAIDA = '49'

S07_CST_COFINSSN = (
        (S07_OUTRAS_OPERACOES_SAIDA, u'Outras operações de saída'),
    )


# COFINSOutr (S06), valores do elemento 'CST' (S07)
S07_OUTRAS_OPERACOES = '99'

S07_CST_COFINSOUTR = (
        (S07_OUTRAS_OPERACOES, u'Outras operações'),
    )


# ISSQN (U01), valores do elemento 'cNatOp' (U09)
U09_TRIBUTACAO_MUNICIPIO = '01'
U09_TRIBUTACAO_FORA_MUNICIPIO = '02'
U09_ISENCAO = '03'
U09_IMUNE = '04'
U09_EXIGIBILIDADE_SUSPENSA_DECISAO_JUDICIAL = '05'
U09_EXIGIBILIDADE_SUSPENSA_PROC_ADM = '06'
U09_NAO_TRIBUTAVEL_OU_NAO_INCIDENTE = '07'
U09_EXPORTACAO_SERVICO = '08'

U09_CNATOP_ISSQN = (
        (U09_TRIBUTACAO_MUNICIPIO, u'Tributação no município'),
        (U09_TRIBUTACAO_FORA_MUNICIPIO, u'Tributação fora do município'),
        (U09_ISENCAO, u'Isenção'),
        (U09_IMUNE, u'Imune'),
        (U09_EXIGIBILIDADE_SUSPENSA_DECISAO_JUDICIAL, u'Exigibilidade suspensa por decisão judicial'),
        (U09_EXIGIBILIDADE_SUSPENSA_PROC_ADM, u'Exigibilidade suspensa por procedimento administrativo'),
        (U09_NAO_TRIBUTAVEL_OU_NAO_INCIDENTE, u'Não tributável ou não incidência'),
        (U09_EXPORTACAO_SERVICO, u'Exportação de serviço'),
    )


# ISSQN (U01), valores do elemento 'indIncFisc' (U10)
U10_SIM = '1'
U10_NAO = '2'

U10_INDINCFISC_ISSQN = (
        (U10_SIM, u'Sim'),
        (U10_NAO, u'Não'),
    )

# MP (WA02), valores do elemento 'cMP' (WA03)
WA03_DINHEIRO = '01'
WA03_CHEQUE = '02'
WA03_CARTAO_CREDITO = '03'
WA03_CARTAO_DEBITO = '04'
WA03_CREDITO_LOJA = '05'
WA03_VALE_ALIMENTACAO = '10'
WA03_VALE_REFEICAO = '11'
WA03_VALE_PRESENTE = '12'
WA03_VALE_COMBUSTIVEL = '13'
WA03_OUTROS = '99'

WA03_CMP_MP = (
        (WA03_DINHEIRO, u'Dinheiro'),
        (WA03_CHEQUE, u'Cheque'),
        (WA03_CARTAO_CREDITO, u'Cartão Crédito'),
        (WA03_CARTAO_DEBITO, u'Cartão Débito'),
        (WA03_CREDITO_LOJA, u'Crédito Loja'),
        (WA03_VALE_ALIMENTACAO, u'Vale Alimentação'),
        (WA03_VALE_REFEICAO, u'Vale Refeição'),
        (WA03_VALE_PRESENTE, u'Vale Presente'),
        (WA03_VALE_COMBUSTIVEL, u'Vale Combustível'),
        (WA03_OUTROS, u'Outros'),
    )


# Anexo 3 - Tabela de Credenciadoras de Cartão de Crédito/Débito
CREDENCIADORAS_CARTAO = (
        # Código da credenciadora, CNPJ, Nome (conforme consta no Anexo 3)
        ('001', '03.106.213/0001-90', u'Administradora de Cartões Sicredi Ltda.'),
        ('002', '03.106.213/0002-71', u'Administradora de Cartões Sicredi Ltda.(filial RS)'),
        ('003', '60.419.645/0001-95', u'Banco American Express S/A - AMEX'),
        ('004', '62.421.979/0001-29', u'BANCO GE - CAPITAL'),
        ('005', '58.160.789/0001-28', u'BANCO SAFRA S/A'),
        ('006', '07.679.404/0001-00', u'BANCO TOPÁZIO S/A'),
        ('007', '17.351.180/0001-59', u'BANCO TRIANGULO S/A'),
        ('008', '04.627.085/0001-93', u'BIGCARD Adm. de Convenios e Serv.'),
        ('009', '01.418.852/0001-66', u'BOURBON Adm. de Cartões de Crédito'),
        ('010', '03.766.873/0001-06', u'CABAL Brasil Ltda.'),
        ('011', '03.722.919/0001-87', u'CETELEM Brasil S/A - CFI'),
        ('012', '01.027.058/0001-91', u'CIELO S/A'),
        ('013', '03.529.067/0001-06', u'CREDI 21 Participações Ltda.'),
        ('014', '71.225.700/0001-22', u'ECX CARD Adm. e Processadora de Cartões S/A'),
        ('015', '03.506.307/0001-57', u'Empresa Bras. Tec. Adm. Conv. Hom. Ltda. - EMBRATEC'),
        ('016', '04.432.048/0001-20', u'EMPÓRIO CARD LTDA'),
        ('017', '07.953.674/0001-50', u'FREEDDOM e Tecnologia e Serviços S/A'),
        ('018', '03.322.366/0001-75', u'FUNCIONAL CARD LTDA.'),
        ('019', '03.012.230/0001-69', u'HIPERCARD Banco Multiplo S/A'),
        ('020', '03.966.317/0001-75', u'MAPA Admin. Conv. e Cartões Ltda.'),
        ('021', '00.163.051/0001-34', u'Novo Pag Adm. e Proc. de Meios Eletrônicos de Pagto. Ltda.'),
        ('022', '43.180.355/0001-12', u'PERNAMBUCANAS Financiadora S/A Crédito, Fin. e Invest.'),
        ('023', '00.904.951/0001-95', u'POLICARD Systems e Serviços Ltda.'),
        ('024', '33.098.658/0001-37', u'PROVAR Negócios de Varejo Ltda.'),
        ('025', '01.425.787/0001-04', u'REDECARD S/A'),
        ('026', '90.055.609/0001-50', u'RENNER Adm. Cartões de Crédito Ltda.'),
        ('027', '03.007.699/0001-00', u'RP Administração de Convênios Ltda.'),
        ('028', '00.122.327/0001-36', u'SANTINVEST S/A Crédito, Financiamento e Investimentos'),
        ('029', '69.034.668/0001-56', u'SODEXHO Pass do Brasil Serviços e Comércio S/A'),
        ('030', '60.114.865/0001-00', u'SOROCRED Meios de Pagamentos Ltda.'),
        ('031', '51.427.102/0004-71', u'Tecnologia Bancária S/A - TECBAN'),
        ('032', '47.866.934/0001-74', u'TICKET Serviços S/A'),
        ('033', '00.604.122/0001-97', u'TRIVALE Administração Ltda.'),
        ('034', '61.071.387/0001-61', u'Unicard Banco Múltiplo S/A - TRICARD'),
        ('999', '0', u'Outros'),)


REDE_TIPOINTER_ETHE = 'ETHE'
REDE_TIPOINTER_WIFI = 'WIFI'

REDE_TIPOINTER_OPCOES = (
        (REDE_TIPOINTER_ETHE, u'Ethernet'),
        (REDE_TIPOINTER_WIFI, u'WiFi'),
    )


REDE_SEG_NONE = 'NONE'
REDE_SEG_WEP = 'WEP'
REDE_SEG_WPA_PERSONAL = 'WPA-PERSONAL'
REDE_SEG_WPA_ENTERPRISE = 'WPA-ENTERPRISE'

REDE_SEG_OPCOES = (
        (REDE_SEG_NONE, u'Nenhuma'),
        (REDE_SEG_WEP, u'WEP'),
        (REDE_SEG_WPA_PERSONAL, u'WPA Personal'),
        (REDE_SEG_WPA_ENTERPRISE, u'WPA Enterprise'),
    )


REDE_TIPOLAN_DHCP = 'DHCP'
REDE_TIPOLAN_PPPOE = 'PPPOE'
REDE_TIPOLAN_IPFIX = 'IPFIX'

REDE_TIPOLAN_OPCOES = (
        (REDE_TIPOLAN_DHCP, u'DHCP'),
        (REDE_TIPOLAN_PPPOE, u'PPPoE'),
        (REDE_TIPOLAN_IPFIX, u'IP fixo'),
    )


REDE_PROXY_NONE = '0'
REDE_PROXY_CONFIGURACAO = '1'
REDE_PROXY_TRANSPARENTE = '2'

REDE_PROXY_OPCOES = (
        (REDE_PROXY_NONE, u'Sem Proxy'),
        (REDE_PROXY_CONFIGURACAO, u'Proxy com configuração'),
        (REDE_PROXY_TRANSPARENTE, u'Proxy transparente'),
    )



# -*- coding: utf-8 -*-
#
# satcomum/ersat.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Este módulo fornece implementações para algumas características descritas na
ER SAT (*Especificação de Requisitos*) que não estão cobertas diretamente neste
projeto nem nos projetos relacionados com o SAT-CF-e.
"""

import re

from . import br
from . import constantes
from . import util


def meio_pagamento(codigo):
    """Obtém a descrição para o código do meio de pagamento.

    :param codigo: Código do meio de pagamento, conforme elemento WA03 ``cMP``.
    :return: Descrição para o código do meio de pagamento.
    :rtype: unicode

    .. sourcecode:: python

        >>> meio_pagamento('01')
        u'Dinheiro'

    """
    return [s for v,s in constantes.WA03_CMP_MP if v == codigo][0]


def dados_qrcode(cfe):
    """Compila os dados que compõem o QRCode do CF-e-SAT, conforme a
    documentação técnica oficial **Guia para Geração do QRCode pelo Aplicativo
    Comercial**, a partir de uma instância de ``ElementTree`` que represente a
    árvore do XML do CF-e-SAT.

    :param cfe: Instância de :py:mod:`xml.etree.ElementTree.ElementTree`.
    :return: String contendo a massa de dados para ser usada ao gerar o QRCode.
    :rtype: str

    Por exemplo, para gerar a imagem do QRCode [#qrcode]_:

    .. sourcecode:: python

        import xml.etree.ElementTree as ET
        import qrcode

        with open('CFe_1.xml', 'r') as fp:
            tree = ET.parse(fp)
            imagem = qrcode.make(dados_qrcode(tree))

    .. [#qrcode] https://pypi.python.org/pypi/qrcode

    """
    infCFe = cfe.getroot().find('./infCFe')
    cnpjcpf_consumidor = infCFe.findtext('dest/CNPJ') or \
            infCFe.findtext('dest/CPF') or ''
    return '|'.join([
            infCFe.attrib['Id'][3:], # remove prefixo "CFe"
            '{}{}'.format(
                    infCFe.findtext('ide/dEmi'),
                    infCFe.findtext('ide/hEmi')),
            infCFe.findtext('total/vCFe'),
            cnpjcpf_consumidor,
            infCFe.findtext('ide/assinaturaQRCODE'),])


class ChaveCFeSAT(object):
    """Representa a **chave de acesso** do CF-e-SAT conforme descrito na
    Especificação de Requisitos SAT, item 4.7. Os campos são definidos assim:

    .. sourcecode:: text

        0  2    6             20 22        31     37     43  --> índice
        |  |    |              |  |         |      |      |
        35 1508 08723218000186 59 900004019 000024 111425 7  --> campos
        |  |    |              |  |         |      |      |
        |  |    |              |  |         |      |      dígito verificador
        |  |    |              |  |         |      |
        |  |    |              |  |         |      código aleatório
        |  |    |              |  |         |
        |  |    |              |  |         número do cupom fiscal
        |  |    |              |  |
        |  |    |              |  número de série do equipamento SAT
        |  |    |              |
        |  |    |              modelo do documento fiscal
        |  |    |
        |  |    cnpj do emitente
        |  |
        |  ano/mês de emissão
        |
        código da UF


    .. sourcecode:: python

        >>> chave = ChaveCFeSAT('CFe35150808723218000186599000040190000241114257')
        >>> chave.codigo_uf
        35
        >>> chave.uf
        'SP'
        >>> chave.mes_emissao
        8
        >>> chave.ano_emissao
        2015
        >>> chave.anomes
        '1508'
        >>> chave.cnpj_emitente
        '08.723.218/0001-86'
        >>> chave.modelo_documento
        '59'
        >>> chave.numero_serie
        '900004019'
        >>> chave.numero_cupom_fiscal
        '000024'
        >>> chave.codigo_aleatorio
        '111425'
        >>> chave.digito_verificador
        '7'

        # chave com mês/ano no mês da Portaria CAT-147, introduzida em 11/2012
        >>> chave = ChaveCFeSAT('CFe35121108723218000186599000040190000241114259')
        >>> chave.mes_emissao
        11
        >>> chave.ano_emissao
        2012

        >>> chave = ChaveCFeSAT('CFe72150808723218000186599000040190000241114250')
        Traceback (most recent call last):
         ...
        ValueError: chave de acesso invalida (codigo UF: '72'): 'CFe72150808723218000186599000040190000241114250'

        >>> # mês/ano inválidos (Portaria CAT-147 introduzida em 11/2012)
        >>> chave = ChaveCFeSAT('CFe35121008723218000186599000040190000241114255')
        Traceback (most recent call last):
         ...
        ValueError: chave de acesso invalida (mes/ano emissao: 10/2012): 'CFe35121008723218000186599000040190000241114255'

        >>> # mês/ano inválidos (Portaria CAT-147 introduzida em 11/2012)
        >>> chave = ChaveCFeSAT('CFe35111208723218000186599000040190000241114250')
        Traceback (most recent call last):
         ...
        ValueError: chave de acesso invalida (mes/ano emissao: 12/2011): 'CFe35111208723218000186599000040190000241114250'

        >>> # mês/ano inválido (mês fora da faixa)
        >>> chave = ChaveCFeSAT('CFe35151308723218000186599000040190000241114251')
        Traceback (most recent call last):
         ...
        ValueError: chave de acesso invalida (mes/ano emissao: 13/2015): 'CFe35151308723218000186599000040190000241114251'

        >>> chave = ChaveCFeSAT('CFe35150808723218000187599000040190000241114259')
        Traceback (most recent call last):
         ...
        ValueError: chave de acesso invalida (CNPJ emitente: '08723218000187'): 'CFe35150808723218000187599000040190000241114259'


    """

    CHAVE_REGEX = re.compile(r'^CFe(?P<campos>\d{44})$')

    CUF = slice(0, 2)

    AAMM = slice(2, 6)

    CNPJ = slice(6, 20)

    MOD = slice(20, 22)

    NSERIESAT = slice(22, 31)

    NCF = slice(31, 37)

    CNF = slice(37, 43)

    CDV = slice(43, None)


    def __init__(self, chave):

        self._chave = None
        self._campos = None

        matcher = ChaveCFeSAT.CHAVE_REGEX.match(chave)
        if matcher:
            campos = matcher.group('campos')
        else:
            raise ValueError('chave de acesso invalida: {!r}'.format(chave))

        digito = util.modulo11(campos[:43])
        if not (digito == int(campos[-1])):
            raise ValueError('digito verificador invalido: '
                    'chave={!r}, digito calculado={!r}'.format(chave, digito))

        if not br.is_codigo_uf(int(campos[ChaveCFeSAT.CUF])):
            raise ValueError('chave de acesso invalida '
                    '(codigo UF: {!r}): {!r}'.format(
                            campos[ChaveCFeSAT.CUF], chave))

        ano = int(campos[ChaveCFeSAT.AAMM][:2]) + 2000
        mes = int(campos[ChaveCFeSAT.AAMM][2:])

        if mes not in xrange(1,13) or \
                (ano < 2012 or (ano == 2012 and mes < 11)):
            # considera válidas apenas as chaves que indicam mês/ano de emissão
            # a partir do início do projeto SAT-CF-e, em novembro/2012
            raise ValueError('chave de acesso invalida '
                    '(mes/ano emissao: {:d}/{:d}): {!r}'.format(
                            mes, ano, chave))

        if not br.is_cnpj(campos[ChaveCFeSAT.CNPJ]):
            raise ValueError('chave de acesso invalida '
                    '(CNPJ emitente: {!r}): {!r}'.format(
                            campos[ChaveCFeSAT.CNPJ], chave))

        self._chave = chave
        self._campos = campos


    def __repr__(self):
        return '{:s}({!r})'.format(self.__class__.__name__, str(self))


    def __unicode__(self):
        return self._chave


    def __str__(self):
        return unicode(self).encode('utf-8')


    @property
    def codigo_uf(self):
        return int(self._campos[ChaveCFeSAT.CUF])


    @property
    def uf(self):
        return br.uf_pelo_codigo(self.codigo_uf)


    @property
    def anomes(self):
        return self._campos[ChaveCFeSAT.AAMM]


    @property
    def ano_emissao(self):
        return int(self._campos[ChaveCFeSAT.AAMM][:2]) + 2000


    @property
    def mes_emissao(self):
        return int(self._campos[ChaveCFeSAT.AAMM][2:])


    @property
    def cnpj_emitente(self):
        return br.as_cnpj(self._campos[ChaveCFeSAT.CNPJ])


    @property
    def modelo_documento(self):
        return self._campos[ChaveCFeSAT.MOD]


    @property
    def numero_serie(self):
        return self._campos[ChaveCFeSAT.NSERIESAT]


    @property
    def numero_cupom_fiscal(self):
        return self._campos[ChaveCFeSAT.NCF]


    @property
    def codigo_aleatorio(self):
        return self._campos[ChaveCFeSAT.CNF]


    @property
    def digito_verificador(self):
        return self._campos[ChaveCFeSAT.CDV]


    def partes(self, num_partes=11):
        """Particiona a chave do CF-e-SAT em uma lista de *n* segmentos.

        :param int num_partes: O número de segmentos (partes) em que os digitos
            da chave do CF-e-SAT serão particionados. **Esse número deverá
            resultar em uma divisão inteira por 44 (o comprimento da chave)**.
            Se não for informado, assume ``11`` partes, comumente utilizado
            para apresentar a chave do CF-e-SAT no extrato.

        :return: Lista de strings contendo a chave do CF-e-SAT particionada.
        :rtype: list

        .. sourcecode:: python

            >>> chave = ChaveCFeSAT('CFe35150461099008000141599000017900000053222424')
            >>> chave.partes()
            ['3515', '0461', '0990', '0800', '0141', '5990', '0001', '7900', '0000', '5322', '2424']
            >>> chave.partes(2)
            ['3515046109900800014159', '9000017900000053222424']
            >>> chave.partes(3)
            Traceback (most recent call last):
             ...
            AssertionError: O numero de partes nao produz um resultado inteiro (partes por 44 digitos): num_partes=3

        """
        assert 44 % num_partes == 0, 'O numero de partes nao produz um '\
                'resultado inteiro (partes por 44 digitos): '\
                'num_partes=%s' % num_partes

        salto = 44 / num_partes
        return [self._campos[n:(n + salto)] for n in range(0, 44, salto)]



# -*- coding: utf-8 -*-
#
# satcomum/br.py
#
# Copyright 2015 Base4 Sistemas Ltda ME
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .util import digitos


class NumeroCNPJError(ValueError):
    pass


class NumeroCPFError(ValueError):
    pass


class NumeroCNPJCPFError(NumeroCNPJError, NumeroCPFError):
    pass


class NumeroCEPError(ValueError):
    pass


class UnidadeFederativaError(ValueError):
    pass


REGIAO_NORTE = u'Norte'
REGIAO_NORDESTE = u'Nordeste'
REGIAO_CENTRO_OESTE = u'Centro Oeste'
REGIAO_SUDESTE = u'Sudeste'
REGIAO_SUL = u'Sul'

_UF_SIGLA = 0
_UF_CODIGO_IBGE = 1
_UF_NOME = 2
_UF_REGIAO = 3

UNIDADES_FEDERACAO = (
        # sigla, código IBGE, nome, região
        ('AC', 12, u'Acre', REGIAO_NORTE),
        ('AL', 27, u'Alagoas', REGIAO_NORDESTE),
        ('AM', 13, u'Amazonas', REGIAO_NORTE),
        ('AP', 16, u'Amapá', REGIAO_NORTE),
        ('BA', 29, u'Bahia', REGIAO_NORDESTE),
        ('CE', 23, u'Ceará', REGIAO_NORDESTE),
        ('DF', 53, u'Distrito Federal', REGIAO_CENTRO_OESTE),
        ('ES', 32, u'Espírito Santo', REGIAO_SUDESTE),
        ('GO', 52, u'Goiás', REGIAO_CENTRO_OESTE),
        ('MA', 21, u'Maranhão', REGIAO_NORDESTE),
        ('MG', 31, u'Minas Gerais', REGIAO_SUDESTE),
        ('MS', 50, u'Mato Grosso do Sul', REGIAO_CENTRO_OESTE),
        ('MT', 51, u'Mato Grosso', REGIAO_CENTRO_OESTE),
        ('PA', 15, u'Pará', REGIAO_NORTE),
        ('PB', 25, u'Paraíba', REGIAO_NORDESTE),
        ('PE', 26, u'Pernambuco', REGIAO_NORDESTE),
        ('PI', 22, u'Piauí', REGIAO_NORDESTE),
        ('PR', 41, u'Paraná', REGIAO_SUL),
        ('RJ', 33, u'Rio de Janeiro', REGIAO_SUDESTE),
        ('RN', 24, u'Rio Grande de Norte', REGIAO_NORDESTE),
        ('RO', 11, u'Rondônia', REGIAO_NORTE),
        ('RR', 14, u'Roraima', REGIAO_NORTE),
        ('RS', 43, u'Rio Grande do Sul', REGIAO_SUL),
        ('SC', 42, u'Santa Catarina', REGIAO_SUL),
        ('SE', 28, u'Sergipe', REGIAO_NORDESTE),
        ('SP', 35, u'São Paulo', REGIAO_SUDESTE),
        ('TO', 17, u'Tocantins', REGIAO_NORTE),)


def uf(sigla):
    """
    Valida a sigla da Unidade Federativa. Se não for uma sigla de UF válida,
    será lançada a exceção :exc:`UnidadeFederativaError`.

    .. sourcecode:: python

        >>> uf('SP')
        >>> uf('sp')
        Traceback (most recent call last):
         ...
        UnidadeFederativaError: Estado (sigla) UF "sp" inexistente

    """
    if not sigla in [s for s, i, n, r in UNIDADES_FEDERACAO]:
        raise UnidadeFederativaError('Estado (sigla) UF "%s" '
                'inexistente' % sigla)


def is_uf(sigla):
    """Uma versão conveniente para usar em testes condicionais. Apenas retorna
    verdadeiro ou falso, conforme o argumento é validado.

    .. sourcecode:: python

        >>> is_uf('')
        False
        >>> is_uf('sp')
        False
        >>> is_uf('SP')
        True

    """
    try:
        uf(sigla)
        return True
    except UnidadeFederativaError:
        pass
    return False


def is_codigo_uf(codigo_ibge):
    """Indica se o código da UF é um código válido.

    .. sourcecode:: python

        >>> is_codigo_uf(35)
        True
        >>> is_codigo_uf(0)
        False

    """
    return codigo_ibge in [i for s, i, n, r in UNIDADES_FEDERACAO]


def uf_pelo_codigo(codigo_ibge):
    """Retorna a UF para o código do IBGE informado.

    .. sourcecode:: python

        >>> uf_pelo_codigo(35)
        'SP'
        >>> uf_pelo_codigo(0)
        Traceback (most recent call last):
         ...
        ValueError: ...

    """
    idx = [i for s, i, n, r in UNIDADES_FEDERACAO].index(codigo_ibge)
    return UNIDADES_FEDERACAO[idx][_UF_SIGLA]


def codigo_ibge_uf(sigla):
    """Retorna o código do IBGE para a UF informada.

    .. sourcecode:: python

        >>> codigo_ibge_uf('SP')
        35
        >>> codigo_ibge_uf('sp')
        Traceback (most recent call last):
         ...
        ValueError: ...
        >>> codigo_ibge_uf('')
        Traceback (most recent call last):
         ...
        ValueError: ...

    """
    idx = [s for s, i, n, r in UNIDADES_FEDERACAO].index(sigla)
    return UNIDADES_FEDERACAO[idx][_UF_CODIGO_IBGE]


def cnpj(numero):
    """Valida um número de CNPJ. O número deverá ser informado como uma string
    contendo 14 dígitos numéricos. Se o número informado for inválido será
    lançada a exceção :exc:`NumeroCNPJError`. Esta implementação da validação
    foi delicadamente copiada de `python-sped <http://git.io/vfuGW>`.

    .. sourcecode:: python

        >>> cnpj('111')
        Traceback (most recent call last):
         ...
        NumeroCNPJError: CNPJ "111" nao possui 14 digitos
        >>> cnpj('08427847000169')
        >>> cnpj('08427847000179') # primeiro digito inválido
        Traceback (most recent call last):
          ...
        NumeroCNPJError: CNPJ "08427847000179" invalido
        >>> cnpj('08427847000168') # segundo digito inválido
        Traceback (most recent call last):
          ...
        NumeroCNPJError: CNPJ "08427847000168" invalido

    """
    _digitos = [int(c) for c in numero if c.isdigit()]

    if len(_digitos) != 14 or len(numero) != 14:
        raise NumeroCNPJError('CNPJ "%s" nao possui 14 digitos' % numero)

    multiplicadores = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]

    soma1 = sum([_digitos[i] * multiplicadores[i+1] for i in range(12)])
    soma2 = sum([_digitos[i] * multiplicadores[i] for i in range(13)])
    digito1 = 11 - (soma1 % 11)
    digito2 = 11 - (soma2 % 11)

    if digito1 >= 10:
        digito1 = 0

    if digito2 >= 10:
        digito2 = 0

    if _digitos[12] != digito1 or _digitos[13] != digito2:
        raise NumeroCNPJError('CNPJ "%s" invalido' % numero)


def is_cnpj(numero, estrito=False):
    """Uma versão conveniente para usar em testes condicionais. Apenas retorna
    verdadeiro ou falso, conforme o argumento é validado.

    :param bool estrito: Padrão ``False``, indica se apenas os dígitos do
        número deverão ser considerados. Se verdadeiro, potenciais caracteres
        que formam a máscara serão removidos antes da validação ser realizada.

    .. sourcecode:: python

        >>> is_cnpj('')
        False
        >>> is_cnpj('08.427.847/0001-69')
        True
        >>> is_cnpj('08.427.847/0001-69', estrito=True)
        False

    """
    try:
        cnpj(digitos(numero) if not estrito else numero)
        return True
    except NumeroCNPJError:
        pass
    return False


def as_cnpj(numero):
    """Formata um número de CNPJ. Se o número não for um CNPJ válido apenas
    retorna o argumento sem qualquer modificação.

    .. sourcecode:: python

        >>> as_cnpj('08427847000169')
        '08.427.847/0001-69'
        >>> as_cnpj('08427847000168')
        '08427847000168'
        >>> as_cnpj('08.427.847/0001-69')
        '08.427.847/0001-69'
        >>> as_cnpj('')
        ''
        >>> as_cnpj('000')
        '000'

    """
    _num = digitos(numero)
    if is_cnpj(_num):
        return '{}.{}.{}/{}-{}'.format(
                _num[:2], _num[2:5], _num[5:8], _num[8:12], _num[12:])
    return numero


def cpf(numero):
    """Valida um número de CPF. O número deverá ser informado como uma string
    contendo 11 dígitos numéricos. Se o número informado for inválido será
    lançada a exceção :exc:`NumeroCPFError`. Esta implementação da validação
    foi delicadamente copiada de `python-sped <http://git.io/vfuGW>`.

    .. sourcecode:: python

        >>> cpf('111')
        Traceback (most recent call last):
         ...
        NumeroCPFError: CPF "111" nao possui 11 digitos
        >>> cpf('11122233396')
        >>> cpf('11122233386') # primeiro digito inválido
        Traceback (most recent call last):
         ...
        NumeroCPFError: CPF "11122233386" invalido
        >>> cpf('11122233395') # segundo digito inválido
        Traceback (most recent call last):
         ...
        NumeroCPFError: CPF "11122233395" invalido

    """
    _digitos = [int(c) for c in numero if c.isdigit()]

    if len(_digitos) != 11 or len(numero) != 11:
        raise NumeroCPFError('CPF "%s" nao possui 11 digitos' % numero)

    multiplicadores = [11, 10, 9, 8, 7, 6, 5, 4, 3, 2]

    soma1 = sum([_digitos[i] * multiplicadores[i+1] for i in range(9)])
    soma2 = sum([_digitos[i] * multiplicadores[i] for i in range(10)])
    digito1 = 11 - (soma1 % 11)
    digito2 = 11 - (soma2 % 11)

    if digito1 >= 10:
        digito1 = 0

    if digito2 >= 10:
        digito2 = 0

    if _digitos[9] != digito1 or _digitos[10] != digito2:
        raise NumeroCPFError('CPF "%s" invalido' % numero)


def is_cpf(numero, estrito=False):
    """Uma versão conveniente para usar em testes condicionais. Apenas retorna
    verdadeiro ou falso, conforme o argumento é validado.

    :param bool estrito: Padrão ``False``, indica se apenas os dígitos do
        número deverão ser considerados. Se verdadeiro, potenciais caracteres
        que formam a máscara serão removidos antes da validação ser realizada.

    .. sourcecode:: python

        >>> is_cpf('')
        False
        >>> is_cpf('111.222.333-96')
        True
        >>> is_cpf('111.222.333-96', estrito=True)
        False

    """
    try:
        cpf(digitos(numero) if not estrito else numero)
        return True
    except NumeroCPFError:
        pass
    return False


def as_cpf(numero):
    """Formata um número de CPF. Se o número não for um CPF válido apenas
    retorna o argumento sem qualquer modificação.

    .. sourcecode:: python

        >>> as_cpf('11122233396')
        '111.222.333-96'
        >>> as_cpf('111.222.333-96')
        '111.222.333-96'
        >>> as_cpf('')
        ''
        >>> as_cpf('000')
        '000'

    """
    _num = digitos(numero)
    if is_cpf(_num):
        return '{}.{}.{}-{}'.format(_num[:3], _num[3:6], _num[6:9], _num[9:])
    return numero


def cnpjcpf(numero):
    """Valida um número de CNPJ ou CPF. Veja :func:`cnpj` e/ou :func:`cpf`.

    .. sourcecode::

        >>> cnpjcpf('08427847000169')
        >>> cnpjcpf('11122233396')
        >>> cnpjcpf('08427847000168') # não é um CNPJ válido, nem um CPF...
        Traceback (most recent call last):
         ...
        NumeroCNPJCPFError: ...

    """
    try:
        cnpj(numero)
    except NumeroCNPJError:
        try:
            cpf(numero)
        except NumeroCPFError:
            raise NumeroCNPJCPFError('numero "%s" nao valida como '
                    'CNPJ nem como CPF' % numero)


def is_cnpjcpf(numero, estrito=False):
    """Uma versão conveniente para usar em testes condicionais. Apenas retorna
    verdadeiro ou falso, conforme o argumento é validado.

    :param bool estrito: Padrão ``False``, indica se apenas os dígitos do
        número deverão ser considerados. Se verdadeiro, potenciais caracteres
        que formam a máscara serão removidos antes da validação ser realizada.

    .. sourcecode:: python

        >>> is_cnpjcpf('')
        False
        >>> is_cnpjcpf('111.222.333-96')
        True
        >>> is_cnpjcpf('08.427.847/0001-69')
        True
        >>> is_cnpjcpf('111.222.333-96', estrito=True)
        False

    """
    _numero = digitos(numero) if not estrito else numero
    try:
        cnpj(_numero)
        return True
    except NumeroCNPJError:
        try:
            cpf(_numero)
            return True
        except NumeroCPFError:
            pass
    return False


def as_cnpjcpf(numero):
    """Formata um número de CNPJ ou CPF. Se o número não for um CNPJ ou CPF
    válidos apenas retorna o argumento sem qualquer modificação.

    .. sourcecode:: python

        >>> as_cnpjcpf('11122233396')
        '111.222.333-96'
        >>> as_cnpjcpf('111.222.333-96')
        '111.222.333-96'
        >>> as_cnpjcpf('08427847000169')
        '08.427.847/0001-69'
        >>> as_cnpjcpf('08.427.847/0001-69')
        '08.427.847/0001-69'
        >>> as_cnpjcpf('')
        ''
        >>> as_cnpjcpf('000')
        '000'

    """
    if is_cnpj(numero):
        return as_cnpj(numero)
    elif is_cpf(numero):
        return as_cpf(numero)
    return numero


def cep(numero):
    """Valida um número de CEP. O número deverá ser informado como uma string
    contendo 8 dígitos numéricos. Se o número informado for inválido será
    lançada a exceção :exc:`NumeroCEPError`.

    .. warning::

        Qualquer string que contenha 8 dígitos será considerada como um CEP
        válido, desde que os dígitos não sejam todos iguais.

    .. sourcecode:: python

        >>> cep('15807250') # OK, este existe mesmo, é válido
        >>> cep('12345678') # OK, considera válido
        >>> cep('123456')
        Traceback (most recent call last):
         ...
        NumeroCEPError: CEP "123456" nao possui 8 digitos

        # 8 digitos mas todos iguais, considera inválido
        >>> cep('11111111')
        Traceback (most recent call last):
         ...
        NumeroCEPError: CEP "11111111" considerado invalido

    """
    _digitos = digitos(numero)

    if len(_digitos) != 8 or len(numero) != 8:
        raise NumeroCEPError('CEP "%s" nao possui 8 digitos' % numero)

    elif _digitos[0] * 8 == _digitos:
        raise NumeroCEPError('CEP "%s" considerado invalido' % numero)


def is_cep(numero, estrito=False):
    """Uma versão conveniente para usar em testes condicionais. Apenas retorna
    verdadeiro ou falso, conforme o argumento é validado.

    :param bool estrito: Padrão ``False``, indica se apenas os dígitos do
        número deverão ser considerados. Se verdadeiro, potenciais caracteres
        que formam a máscara serão removidos antes da validação ser realizada.

    .. sourcecode:: python

        >>> is_cep('')
        False
        >>> is_cep('15807-250')
        True
        >>> is_cpf('15807-250', estrito=True)
        False

    """
    try:
        cep(digitos(numero) if not estrito else numero)
        return True
    except NumeroCEPError:
        pass
    return False


def as_cep(numero):
    """Formata um número de CEP. Se o argumento não for um CEP válido apenas
    retorna o argumento sem qualquer modificação.

    .. sourcecode:: python

        >>> as_cep('15807250')
        '15807-250'
        >>> as_cep('15807-250')
        '15807-250'
        >>> as_cep('')
        ''
        >>> as_cep('123456')
        '123456'

    """
    _numero = digitos(numero)
    if is_cep(_numero):
        return '{}-{}'.format(_numero[:5], _numero[5:])
    return numero
